/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#if !defined(AFX_TASKBARBUTTON_H__80A0FBD2_FB89_4557_822D_0E5BB681EC99__INCLUDED_)
#define AFX_TASKBARBUTTON_H__80A0FBD2_FB89_4557_822D_0E5BB681EC99__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

typedef vector<HWND> hWndGroupList;
typedef vector<HWND>::iterator hWndGroupListIterator;
 
class TaskbarButton  
{
public:

	friend class Taskbar;

	TaskbarButton( Taskbar *settaskbar, HWND hWnd );
	virtual ~TaskbarButton();

private:

	Taskbar *taskbar;

	//--------------------------------------------------
	//Grouping
	//--------------------------------------------------
	hWndGroupList listhWndGroup;

	void AddToGroup( HWND hWndAddTask )
	{
		listhWndGroup.insert(listhWndGroup.end(), hWndAddTask);
	
		if ( !strTooltip.empty() )
			CreateTooltip(true);
	};

	bool SearchInGroup( HWND hWndSearchTask )
	{
		for(hWndGroupListIterator hglit = listhWndGroup.begin(); hglit != listhWndGroup.end(); ++hglit)
		{
			if ( *hglit == hWndSearchTask )
				return true;
		}
		return false;
	};

	bool RemoveFromGroup( HWND hWndRemoveTask )
	{
		for(hWndGroupListIterator hglit = listhWndGroup.begin(); hglit != listhWndGroup.end(); ++hglit)
		{
			if ( (*hglit) == hWndRemoveTask )
			{
				delete (*hglit);
				listhWndGroup.erase( hglit );

				hWndButtonTask = (*listhWndGroup.begin());
				return true;
			}
		}
		return false;
	};
	//--------------------------------------------------

	HWND hWndButtonTask;

	string strClassName;
	string strCaption;
	string strTooltip;

	int iButtonX;
	int iButtonY;
	int iButtonWidth;
	int iButtonHeight;

	int iPositionInTaskbar;

	HMONITOR hMonitorCurrentMonitor;
	bool bMonitorHidden;

	void SetButtonLayout( int x, int y, int width, int height );
	
	bool HasPoint( int xP, int yP );

	void CreateTooltip( bool grouped = false );
	
	int GetCurrentState();
	string GetWindowCaption( HWND check_hTask = NULL );

	int iCurrentState;

	bool bIsFlashing; //True Timer is running
	bool bFlashState; //True Flashing, False Normal

	void SetFlashState(int state)
	{
		if ( state == -1 )
		{
			bFlashState = !bFlashState;
		}
		else if ( state == 1 )
		{
			bFlashState = true;
			bIsFlashing = true;
		}
		else
		{
			bFlashState = false;
			bIsFlashing = false;
		}
	}

	bool IsFlashing() { return bIsFlashing; }

	bool bIsHover;
	bool bIsPressed;
	bool bIsScrolling;

	bool Update( bool needRedraw = false );

	void ButtonClick( int x, int y, int action, bool minimize = false );
	
	void Select( bool minimize );
	
	void Menu();
	void GroupMenu();

	void Paint( HDC desthDC );
	void OverlayButtonPainter( HDC desthDC, int x, int y, int width, int height, int buttonType );
	
	void ScrollPainter( HDC desthDC, int x, int y, int width, int height, const string &drawText, int buttonType );

protected:

	HICON hIcon;

};

#endif // !defined(AFX_TASKBARBUTTON_H__80A0FBD2_FB89_4557_822D_0E5BB681EC99__INCLUDED_)
